package cn.com.ctop.job.kuaishou.controller;

import cn.com.ctop.job.kuaishou.data.service.*;
import cn.com.ctop.job.kuaishou.data.utils.Check;
import cn.com.ctop.job.kuaishou.data.utils.DateUtils;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.google.common.collect.Lists;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;

/**
 * zian Y
 * 2022/2/9
 **/

@RestController
@RequestMapping("/kuaishou/kuaishouController")
@Api(tags = "快手")
@Slf4j
public class KuaishouController {
    @Autowired
    private IOauthTokenService tokenService;

    @Autowired
    private IKuaishouVideoListService videoListService;

    @Autowired
    private IAdUnitReportService adUnitReportService;

    @Autowired
    private IKuaishouUnitReportHourlyService unitReportHourlyService;



    @ApiOperation(value = "根据账户获取视频素材-账户下全部视频素材", notes = "根据账户获取视频素材-账户下全部视频素材")
    @PostMapping(value = "/getKuaishouVideoByAccountId")
    public Map<String, String> getKuaishouVideoByAccountId(@RequestBody JSONObject jsonObject) {
        Long accountId = jsonObject.getLong("accountId");
        JSONArray dataList = jsonObject.getJSONArray("photoIds");
        List<String> materialIds = Check.isNull(dataList) ? null : JSONArray.parseObject(dataList.toJSONString(), List.class);
        String token = tokenService.getByAccountId(accountId);
        return videoListService.getKuaishouVideoByAccountId(accountId, token, materialIds, 1);
    }


    @Autowired
    private IKuaishouUnitAudienceAgeReportDailyService unitReportDailyService;


    @ApiOperation(value = "根据账户获取视频素材-账户下全部视频素材", notes = "根据账户获取视频素材-账户下全部视频素材")
    @GetMapping(value = "/KuaishouUnitAudienceReport")
    public void KuaishouUnitAudienceReportJob(String startDate, String endDate) throws Exception {
        List<String> allDates = DateUtils.getAllDatesOfTwoTimes(startDate, endDate);

        for (String date : allDates) {
            List<Long> accountIds = unitReportDailyService.selectAccountIdsByTime(date);
            for (Long accountId : accountIds) {
                String token = tokenService.getByAccountId(accountId);
                if (Check.isNull(token)) {
                    continue;
                }
                //广告计划 ID 集
                List<JSONObject> campaignArray = unitReportDailyService.selectPlanIdsByAccountIdAndTime(accountId, date);
                if (Check.isNull(campaignArray)) {
                    log.error("未查询到计划ID");
                    continue;
                }
                //广告组 ID 集
                for (JSONObject obj : campaignArray) {
                    Long campaignId = obj.getLong("campaignId");
                    List<Long> unitIds = unitReportDailyService.selectUnitIdsByAccountIdAndTime(accountId, campaignId, date);
                    if (Check.isNull(unitIds)) {
                        continue;
                    }
                    if (unitIds.size() > 59) {
                        List<List<Long>> unitIdArray = Lists.partition(unitIds, 59);
                        for (List<Long> list : unitIdArray) {
                            unitReportDailyService.standardCrowdAnalysisReport(accountId, token, date, campaignId, list);
                        }
                    } else {
                        unitReportDailyService.standardCrowdAnalysisReport(accountId, token, date, campaignId, unitIds);
                    }
                }
            }
        }
    }

    @ApiOperation(value = "获取快手当天-广告组信息、组时报", notes = "更新快手当天-广告组信息、组时报")
    @GetMapping(value = "/getKuaishouAdUnitRealTime")
    public void getKuaishouVideoByAccountId(@RequestParam(name="accountId") long accountId) {
        String nowDate = DateUtils.getNowDate("yyyy-MM-dd");
        String token = tokenService.getByAccountId(accountId);
        //更新 组
        adUnitReportService.getAdUnitReport(accountId,token, nowDate, nowDate, 1);
        //更新 组时报
        unitReportHourlyService.getUnitReportHour(accountId, token, nowDate, nowDate, 1);
    }




}