package cn.com.ctop.job.kuaishou.data.service.impl;

import cn.com.ctop.job.kuaishou.data.constant.KuaishouConstant;
import cn.com.ctop.job.kuaishou.data.entity.KuaishouAccountReportDaily;
import cn.com.ctop.job.kuaishou.data.mapper.KuaishouAccountReportDailyMapper;
import cn.com.ctop.job.kuaishou.data.service.IKuaishouAccountReportDailyService;
import cn.com.ctop.job.kuaishou.data.utils.DateUtils;
import cn.com.ctop.job.kuaishou.data.utils.HttpUtils;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 快手-账户日报
 *
 * @author jeecg-boot 2021-09-07
 * @version V1.0
 */
@Slf4j
@Service
public class KuaishouAccountReportDailyServiceImpl extends ServiceImpl<KuaishouAccountReportDailyMapper, KuaishouAccountReportDaily> implements IKuaishouAccountReportDailyService {

    @Autowired
    private KuaishouAccountReportDailyMapper accountReportDailyMapper;

    @Value("${api.kuaishou.postUrl}")
    private String postUrl;

    @Override
    public void getAdvertiserReportDaily(Long advertiserId, String accessToken, String startDate, String endDate, int page)throws Exception {
        log.info("获取账户日报，账户id：{}，开始时间：{}，结束时间：{}", advertiserId, startDate, endDate);
        String url = postUrl + KuaishouConstant.ACCOUNT_REPORT;
        Map<String, String> headers = new HashMap<>();
        headers.put("Access-Token", accessToken);
        Map<String, Object> param = new HashMap<>();
        param.put("start_date", startDate);
        param.put("end_date", endDate);
        param.put("advertiser_id", advertiserId);
        param.put("temporal_granularity", "DAILY");
        param.put("page_size", 2000);
        param.put("page", page);
        String result = HttpUtils.httpPostRequest(url, param, headers);
        JSONObject resultJson = JSONObject.parseObject(result);
     //   System.err.println(resultJson);
        Integer code = resultJson.getInteger("code");
        String message = resultJson.getString("message");
        if (null == code || code != 0) {
            log.error("获取快手广告主日报表异常:{},accountId:{}", message, advertiserId);
            return;
        }
        JSONArray details = resultJson.getJSONObject("data").getJSONArray("details");
        if (null == details || details.size() <= 0) {
            log.error("快手日报返回详情为空，accountId：{}", advertiserId);
            return;
        }
        List<KuaishouAccountReportDaily> addList = new ArrayList<>();
        for (int i = 0; i < details.size(); i++) {
            JSONObject returnJson = details.getJSONObject(i);
            KuaishouAccountReportDaily reportDaily = KuaishouAccountReportDaily.getKuaishouAccountReportDailyInfo(returnJson);
            reportDaily.setAdvertiserId(advertiserId);
            addList.add(reportDaily);
        }
        accountReportDailyMapper.replaceBatch(addList);
        sendMessage(advertiserId, startDate, endDate, "daily");
        log.info("账户日报数据获取完成：accountId：{}", advertiserId);
    }

    @Override
    public void sendMessage(Long accountId, String startDate, String endDate, String date_type) {
        String url = KuaishouConstant.SEND_URL;
        Map<String, Object> param = new HashMap<>();
        param.put("start_date", DateUtils.getDateInteger(startDate));
        param.put("end_date", DateUtils.getDateInteger(endDate));
        param.put("account_id", accountId);
        param.put("date_type", date_type);
        String result = HttpUtils.httpPostRequest(url, param, null);
        System.err.println(result);
    }
}
